@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     cubic-iter-2.cpp
- Revision: $Id: cubic-iter-2.cpp 299 2012-03-21 16:08:40Z kulshres $
+ Revision: $Id: cubic-iter-2.cpp 527 2014-07-15 14:09:31Z kulshres $
  Contents: example for cubic lighthouse example of Griewank's Book
            using iterative solvers
            (output of z_k and dz_k for fixed t)
@@ -69,14 +69,14 @@ int main() {
     size_t tape_stats[STAT_SIZE];
     tapestats(tag,tape_stats);
 
-    fprintf(stdout,"\n    independents            %d\n",tape_stats[NUM_INDEPENDENTS]);
-    fprintf(stdout,"    dependents              %d\n",tape_stats[NUM_DEPENDENTS]);
-    fprintf(stdout,"    operations              %d\n",tape_stats[NUM_OPERATIONS]);
-    fprintf(stdout,"    operations buffer size  %d\n",tape_stats[OP_BUFFER_SIZE]);
-    fprintf(stdout,"    locations buffer size   %d\n",tape_stats[LOC_BUFFER_SIZE]);
-    fprintf(stdout,"    constants buffer size   %d\n",tape_stats[VAL_BUFFER_SIZE]);
-    fprintf(stdout,"    maxlive                 %d\n",tape_stats[NUM_MAX_LIVES]);
-    fprintf(stdout,"    valstack size           %d\n\n",tape_stats[TAY_STACK_SIZE]);
+    fprintf(stdout,"\n    independents            %zu\n",tape_stats[NUM_INDEPENDENTS]);
+    fprintf(stdout,"    dependents              %zu\n",tape_stats[NUM_DEPENDENTS]);
+    fprintf(stdout,"    operations              %zu\n",tape_stats[NUM_OPERATIONS]);
+    fprintf(stdout,"    operations buffer size  %zu\n",tape_stats[OP_BUFFER_SIZE]);
+    fprintf(stdout,"    locations buffer size   %zu\n",tape_stats[LOC_BUFFER_SIZE]);
+    fprintf(stdout,"    constants buffer size   %zu\n",tape_stats[VAL_BUFFER_SIZE]);
+    fprintf(stdout,"    maxlive                 %zu\n",tape_stats[NUM_MAX_LIVES]);
+    fprintf(stdout,"    valstack size           %zu\n\n",tape_stats[TAY_STACK_SIZE]);
 
     /*--------------------------------------------------------------------------*/
     x[1] = t;
