@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     convolute.c
- Revision: $Id: convolut.c 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: convolut.c 106 2010-06-29 17:19:50Z kulshres $
  Contents: Convolution routines (used by ho_rev.mc)
  
  Copyright (c) Andrea Walther, Andreas Griewank, Andreas Kowarz, 
@@ -13,7 +13,7 @@
  
 ----------------------------------------------------------------------------*/
 
-#include <convolut.h>
+#include <adolc/convolut.h>
 
 BEGIN_C_DECLS
 
