@@ -1,11 +1,12 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     sparse/sparsedrivers.cpp
- Revision: $Id: sparsedrivers.cpp 373 2012-12-07 12:24:52Z kulshres $
+ Revision: $Id: sparsedrivers.cpp 527 2014-07-15 14:09:31Z kulshres $
  Contents: "Easy To Use" C++ interfaces of SPARSE package
  
- Copyright (c) Andrea Walther
-  
+
+ Copyright (c) Andrea Walther, Benjamin Letschert, Kshitij Kulshreshtha
+
  This file is part of ADOL-C. This software is provided as open source.
  Any use, reproduction, or distribution of the software constitutes 
  recipient's acceptance of the terms of the accompanying license file.  
@@ -36,6 +37,7 @@ using namespace ColPack;
 
 using namespace std;
 
+
 /****************************************************************************/
 /*******       sparse Jacobains, separate drivers             ***************/
 /****************************************************************************/
@@ -71,7 +73,7 @@ int jac_pat(
     if (crs == NULL) {
         fprintf(DIAG_OUT,"ADOL-C user error in jac_pat(...) : "
                 "parameter crs may not be NULL !\n");
-        exit(-1);
+        adolc_exit(-1,"",__func__,__FILE__,__LINE__);
     } else
         for (i=0; i<depen; i++)
             crs[i] = NULL;
@@ -114,7 +116,7 @@ void generate_seed_jac
 ) 
 #if HAVE_LIBCOLPACK
 {
-  int dummy, i, j;
+  int dummy;
 
     BipartiteGraphPartialColoringInterface *g = new BipartiteGraphPartialColoringInterface(SRC_MEM_ADOLC, JP, m, n);
 
@@ -130,7 +132,7 @@ void generate_seed_jac
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -163,7 +165,7 @@ int hess_pat(
     if (crs == NULL) {
         fprintf(DIAG_OUT,"ADOL-C user error in hess_pat(...) : "
                 "parameter crs may not be NULL !\n");
-        exit(-1);
+        adolc_exit(-1,"",__func__,__FILE__,__LINE__);
     } else
         for (i=0; i<indep; i++)
             crs[i] = NULL;
@@ -196,7 +198,7 @@ void generate_seed_hess
 )
 #if HAVE_LIBCOLPACK 
 {
-  int seed_rows, i, j;
+  int seed_rows;
 
   GraphColoringInterface *g = new GraphColoringInterface(SRC_MEM_ADOLC, HP, n);
 
@@ -211,7 +213,7 @@ void generate_seed_hess
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -262,8 +264,7 @@ int sparse_jac(
     int i;
     unsigned int j;
     SparseJacInfos sJinfos;
-    int dummy;
-    int ret_val;
+    int ret_val = 0;
     BipartiteGraphPartialColoringInterface *g;
     TapeInfos *tapeInfos;
     JacobianRecovery1D *jr1d;
@@ -413,7 +414,7 @@ int sparse_jac(
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -483,7 +484,7 @@ int sparse_hess(
 	    memcpy(&ADOLC_CURRENT_TAPE_INFOS, tapeInfos, sizeof(TapeInfos));
             if (indep != ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.indep) {
                 fprintf(DIAG_OUT,"ADOL-C Error: wrong number of independents stored in hessian pattern.\n");
-                exit(-1);
+                adolc_exit(-1,"",__func__,__FILE__,__LINE__);
             }
 	    deepcopy_HP(&sHinfos.HP,ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.HP,indep);	    
 	  }
@@ -641,7 +642,7 @@ int sparse_hess(
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -680,7 +681,7 @@ void set_HP(
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if sparse configuration option was used\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -690,7 +691,6 @@ void get_HP(
     unsigned int *** HP)
 #ifdef SPARSE
 {
-    SparseHessInfos sHinfos;
     TapeInfos *tapeInfos;
 
     ADOLC_OPENMP_THREAD_NUMBER;
@@ -703,7 +703,7 @@ void get_HP(
 #else
 {
     fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if sparse configuration option was used\n", __FUNCTION__);
-    exit(-1);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
 }
 #endif
 
@@ -774,7 +774,7 @@ int bit_vector_propagation(
       
         if (( tight_mode ) && ( basepoint == NULL )) {
             fprintf(DIAG_OUT, "ADOL-C error in jac_pat(...) :  supply basepoint x for tight mode.\n");
-            exit(-1);
+            adolc_exit(-1,"",__func__,__FILE__,__LINE__);
         }
 
         /* indep partial derivatives for the whole Jacobian */
@@ -800,10 +800,10 @@ int bit_vector_propagation(
 
         if ( ! (indep_blocks_flags = (unsigned char*)
                                      calloc(i_blocks_per_strip, sizeof(char)) ) ) {
-            fprintf(DIAG_OUT, "ADOL-C error, "__FILE__
+            fprintf(DIAG_OUT, "ADOL-C error, " __FILE__
                     ":%i : \njac_pat(...) unable to allocate %i bytes !\n",
                     __LINE__, (int)(i_blocks_per_strip*sizeof(char)));
-            exit(-1);
+            adolc_exit(-1,"",__func__,__FILE__,__LINE__);
         }
 
         seed        = myalloc2_ulong(indep, p_stripmine);
@@ -877,10 +877,10 @@ int bit_vector_propagation(
                 if ((k > 0 ) || ( strip_idx == 0 )) {
                     if ( ! (crs[j] = (unsigned int*)realloc(crs[j],
                                             (k_old+k+1)*sizeof(unsigned int))) ) {
-                        fprintf(DIAG_OUT, "ADOL-C error, "__FILE__
+                        fprintf(DIAG_OUT, "ADOL-C error, " __FILE__
                                  ":%i : \njac_pat(...) unable to allocate %i bytes !\n",
                                 __LINE__, (int)((k_old+k+1)*sizeof(unsigned int)));
-                        exit(-1);
+                        adolc_exit(-1,"",__func__,__FILE__,__LINE__);
                     }
                     if ( strip_idx == 0 )
                         crs[j][0]  = 0;
@@ -929,10 +929,10 @@ int bit_vector_propagation(
         /* allocate memory --------------------------------------------------- */
         if ( ! (indep_blocks_flags = (unsigned char*)calloc(indep,
                                      sizeof(unsigned char)) ) ) {
-            fprintf(DIAG_OUT, "ADOL-C error, "__FILE__
+            fprintf(DIAG_OUT, "ADOL-C error, " __FILE__
                     ":%i : \njac_pat(...) unable to allocate %i bytes !\n",
                     __LINE__, (int)(indep*sizeof(unsigned char)));
-            exit(-1);
+            adolc_exit(-1,"",__func__,__FILE__,__LINE__);
         }
 
         seed        = myalloc2_ulong(q_stripmine, depen);
@@ -945,7 +945,7 @@ int bit_vector_propagation(
             if ( basepoint == NULL ) {
                 fprintf(DIAG_OUT, "ADOL-C error in jac_pat(..) :  ");
                 fprintf(DIAG_OUT, "no basepoint x for tight mode supplied.\n");
-                exit(-1);
+                adolc_exit(-1,"",__func__,__FILE__,__LINE__);
             }
 
             rc = zos_forward(tag, depen, indep, 1, basepoint, valuepoint);
@@ -1009,10 +1009,10 @@ int bit_vector_propagation(
                     k += *i_b_flags++;
 
                 if ( ! (crs[d_bl_idx] = (unsigned int*)malloc((k+1)*sizeof(unsigned int))) ) {
-                    fprintf(DIAG_OUT, "ADOL-C error, "__FILE__
+                    fprintf(DIAG_OUT, "ADOL-C error, " __FILE__
                             ":%i : \njac_pat(...) unable to allocate %i bytes !\n",
                             __LINE__, (int)((k+1)*sizeof(unsigned int)));
-                    exit(-1);
+                    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
                 }
                 crs[d_bl_idx][0] = k; /* number of non-zero indep. blocks */
                 k=1;
@@ -1054,7 +1054,6 @@ BEGIN_C_DECLS
 void freeSparseJacInfos(double *y, double **B, unsigned int **JP, void *g, 
 			void *jr1d, int seed_rows, int seed_clms, int depen)
 {
-    int i;
     if(y)
       myfree1(y);
 
@@ -1083,8 +1082,6 @@ void freeSparseHessInfos(double **Hcomp, double ***Xppp, double ***Yppp, double
                          double **Upp, unsigned int **HP,
                          void *g, void *hr, int p, int indep)
 {
-    int i;
-
     if(Hcomp)
       myfree2(Hcomp);
 
@@ -1117,6 +1114,208 @@ void freeSparseHessInfos(double **Hcomp, double ***Xppp, double ***Yppp, double
 
 END_C_DECLS
 
+#include <adolc/adtl.h>
+
+namespace adtl {
+
+#ifdef SPARSE
+SparseJacInfos sJinfos
+     = { NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, 0 };
+#endif
+
+int ADOLC_get_sparse_jacobian( func_ad *const fun,
+			       int n, int m, int repeat, double* basepoints,
+			       int *nnz, unsigned int **rind,
+			       unsigned int **cind, double **values)
+#if HAVE_LIBCOLPACK
+{
+    int i;
+    unsigned int j;
+    int dummy;
+    int ret_val = -1;
+    if (!repeat) {
+    freeSparseJacInfos(sJinfos.y, sJinfos.B, sJinfos.JP, sJinfos.g, sJinfos.jr1d, sJinfos.seed_rows, sJinfos.seed_clms, sJinfos.depen);
+    //setNumDir(n);
+    setMode(ADTL_INDO);
+    {
+	adouble *x, *y;
+	x = new adouble[n];
+	y = new adouble[m];
+    for (i=0; i < n ; i++){
+      x[i] = basepoints[i];
+      //x[i].setADValue(i,1);
+    }
+    ret_val = ADOLC_Init_sparse_pattern(x,n,0);
+
+    ret_val = (*fun)(n,x,m,y);
+
+    if (ret_val < 0) {
+       printf(" ADOL-C error in tapeless sparse_jac() \n");
+       return ret_val;
+    }
+
+    ret_val = ADOLC_get_sparse_pattern(y, m, sJinfos.JP );
+	delete[] x;
+	delete[] y;
+    }
+    sJinfos.depen = m;
+    sJinfos.nnz_in = 0;
+    for (i=0;i<m;i++) {
+       for (j=1;j<=sJinfos.JP[i][0];j++)
+          sJinfos.nnz_in++;
+    }
+      *nnz = sJinfos.nnz_in;
+      /* sJinfos.Seed is memory managed by ColPack and will be deleted
+       * along with g. We only keep it in sJinfos for the repeat != 0 case */
+      BipartiteGraphPartialColoringInterface *g;
+      JacobianRecovery1D *jr1d;
+
+      g = new BipartiteGraphPartialColoringInterface(SRC_MEM_ADOLC, sJinfos.JP, m, n);
+      jr1d = new JacobianRecovery1D;
+
+      g->GenerateSeedJacobian(&(sJinfos.Seed), &(sJinfos.seed_rows),
+                                &(sJinfos.seed_clms), "SMALLEST_LAST","COLUMN_PARTIAL_DISTANCE_TWO");
+      sJinfos.seed_rows = m;
+
+      sJinfos.B = myalloc2(sJinfos.seed_rows,sJinfos.seed_clms);
+      sJinfos.y = myalloc1(m);
+
+      sJinfos.g = (void *) g;
+      sJinfos.jr1d = (void *) jr1d;
+
+    if (sJinfos.nnz_in != *nnz) {
+        printf(" ADOL-C error in sparse_jac():"
+               " Number of nonzeros not consistent,"
+               " repeat call with repeat = 0 \n");
+        return -3;
+    }
+
+    }
+//  ret_val = fov_forward(tag, depen, indep, sJinfos.seed_clms, basepoint, sJinfos.Seed, sJinfos.y, sJinfos.B);
+    setNumDir(sJinfos.seed_clms);
+    setMode(ADTL_FOV);
+    {
+	adouble *x, *y;
+	x = new adouble[n];
+	y = new adouble[m];
+
+    for (i=0; i < n ; i++){
+      x[i] = basepoints[i];
+      for (j=0; j < sJinfos.seed_clms; j++)
+	  x[i].setADValue(j,sJinfos.Seed[i][j]);
+    }
+
+    ret_val = (*fun)(n,x,m,y);
+
+    for (i=0;i<m;i++)
+       for (j=0; j< sJinfos.seed_clms;j++)
+          sJinfos.B[i][j] = y[i].getADValue(j);
+
+	delete[] x;
+	delete[] y;
+    }
+	/* recover compressed Jacobian => ColPack library */
+
+      if (*values != NULL)
+       free(*values);
+      if (*rind != NULL)
+       free(*rind);
+      if (*cind != NULL)
+       free(*cind);
+     BipartiteGraphPartialColoringInterface *g;
+     JacobianRecovery1D *jr1d;
+     g = (BipartiteGraphPartialColoringInterface*)sJinfos.g;
+     jr1d = (JacobianRecovery1D*)sJinfos.jr1d;
+     jr1d->RecoverD2Cln_CoordinateFormat_unmanaged(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+
+    //delete g;
+    //delete jr1d;
+
+    return ret_val;
+
+}
+#else
+{
+    fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
+}
+#endif
+
+#if 0
+int ADOLC_get_sparse_jacobian(int n, int m, adouble *x, int *nnz, unsigned int *rind, unsigned int *cind, double *values)
+#if HAVE_LIBCOLPACK
+{
+    int i;
+    unsigned int j;
+    SparseJacInfos sJinfos;
+    int dummy;
+    int ret_val = -1;
+    BipartiteGraphPartialColoringInterface *g;
+    JacobianRecovery1D *jr1d;
+
+    ret_val = ADOLC_get_sparse_pattern(x, m, sJinfos.JP );
+
+    sJinfos.depen = m;
+    sJinfos.nnz_in = 0;
+    for (i=0;i<m;i++) {
+       for (j=1;j<=sJinfos.JP[i][0];j++)
+          sJinfos.nnz_in++;
+    }
+      *nnz = sJinfos.nnz_in;
+      /* sJinfos.Seed is memory managed by ColPack and will be deleted
+       * along with g. We only keep it in sJinfos for the repeat != 0 case */
+
+      g = new BipartiteGraphPartialColoringInterface(SRC_MEM_ADOLC, sJinfos.JP, m, n);
+      jr1d = new JacobianRecovery1D;
+
+      g->GenerateSeedJacobian(&(sJinfos.Seed), &(sJinfos.seed_rows),
+                                &(sJinfos.seed_clms), "SMALLEST_LAST","COLUMN_PARTIAL_DISTANCE_TWO");
+      sJinfos.seed_rows = m;
+
+      sJinfos.B = myalloc2(sJinfos.seed_rows,sJinfos.seed_clms);
+      sJinfos.y = myalloc1(m);
+
+      sJinfos.g = (void *) g;
+      sJinfos.jr1d = (void *) jr1d;
+
+    if (sJinfos.nnz_in != *nnz) {
+        printf(" ADOL-C error in sparse_jac():"
+               " Number of nonzeros not consistent,"
+               " repeat call with repeat = 0 \n");
+        return -3;
+    }
+
+//  ret_val = fov_forward(tag, depen, indep, sJinfos.seed_clms, basepoint, sJinfos.Seed, sJinfos.y, sJinfos.B);
+    for (i=0;i<m;i++)
+       for (j=0; j< sJinfos.seed_clms;j++)
+          sJinfos.B[i][j] = x[i].getADValue(j);
+
+    /* recover compressed Jacobian => ColPack library */
+
+      if (values != NULL)
+       free(values);
+      if (rind != NULL)
+       free(rind);
+      if (cind != NULL)
+       free(cind);
+     jr1d->RecoverD2Cln_CoordinateFormat_unmanaged(g, sJinfos.B, sJinfos.JP, &rind, &cind, &values);
+
+    delete g;
+    delete jr1d;
+
+    return ret_val;
+
+}
+#else
+{
+    fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if linked with ColPack\n", __FUNCTION__);
+    adolc_exit(-1,"",__func__,__FILE__,__LINE__);
+}
+#endif
+#endif
+
+}
+
 /****************************************************************************/
 /*                                                               THAT'S ALL */
 
